@@ -55,6 +55,7 @@
    ]).
 
 -include_lib("cloudi_x_exometer_core/include/cloudi_x_exometer.hrl").
+-include_lib("cloudi_x_hut/include/cloudi_x_hut.hrl").
 
 -define(DEFAULT_HOST, "localhost").
 -define(DEFAULT_PORT, 4242).
@@ -73,12 +74,10 @@
 %% calendar:datetime_to_gregorian_seconds({{1970,1,1},{0,0,0}}).
 -define(UNIX_EPOCH, 62167219200).
 
--include("cloudi_x_exometer_log.hrl").
-
 %% Probe callbacks
 
 exometer_init(Opts) ->
-    ?info("Exometer OpenTSDB Reporter; Opts: ~p~n", [Opts]),
+    ?log(info, "Exometer OpenTSDB Reporter; Opts: ~p~n", [Opts]),
     {Host, Port} = get_opt(host, Opts, {?DEFAULT_HOST, ?DEFAULT_PORT}),
     ReconnectInterval = get_opt(reconnect_interval, Opts, ?RECONNECT_INTERVAL) * 1000,
     ConnectTimeout = get_opt(connect_timeout, Opts, ?DEFAULT_CONNECT_TIMEOUT),
@@ -96,14 +95,14 @@ exometer_init(Opts) ->
         {ok, Sock} ->
             {ok, State#st{socket = Sock}};
         {error, _} = Error ->
-            ?warning("Exometer opentsdb connection failed; ~p. Retry in ~p~n", [Error, ReconnectInterval]),
+            ?log(warning, "Exometer opentsdb connection failed; ~p. Retry in ~p~n", [Error, ReconnectInterval]),
             prepare_reconnect(),
             {ok, State}
     end.
 
 %% Exometer report when no opentsdb socket connection exists.
 cloudi_x_exometer_report(_Metric, _DataPoint, _Extra, _Value, St) when St#st.socket =:= undefined ->
-    ?warning("Report metric: No connection. Value lost~n"),
+    ?log(warning, "Report metric: No connection. Value lost~n"),
     {ok, St};
 
 %% Format a opentsdb output. Each entry is one measurement plus key/value attributes.
@@ -139,11 +138,11 @@ exometer_unsubscribe(_Metric, _DataPoint, _Extra, St) ->
     {ok, St }.
 
 exometer_call(Unknown, From, St) ->
-    ?info("Unknown call ~p from ~p", [Unknown, From]),
+    ?log(info, "Unknown call ~p from ~p", [Unknown, From]),
     {ok, St}.
 
 exometer_cast(Unknown, St) ->
-    ?info("Unknown cast: ~p", [Unknown]),
+    ?log(info, "Unknown cast: ~p", [Unknown]),
     {ok, St}.
 
 
@@ -151,17 +150,17 @@ cloudi_x_exometer_info({exometer_callback, prepare_reconnect}, #st{reconnect_int
     reconnect_after(Int),
     {ok, St};
 cloudi_x_exometer_info({exometer_callback, reconnect}, St) ->
-    ?info("Reconnecting: ~p~n", [St]),
+    ?log(info, "Reconnecting: ~p~n", [St]),
     case connect_opentsdb(St) of
         {ok, NSt} ->
             {ok, NSt};
         Err ->
-            ?warning("Could not reconnect: ~p~n", [Err]),
+            ?log(warning, "Could not reconnect: ~p~n", [Err]),
             prepare_reconnect(),
             {ok, St}
     end;
 cloudi_x_exometer_info(Unknown, St) ->
-    ?info("Unknown info: ~p", [Unknown]),
+    ?log(info, "Unknown info: ~p", [Unknown]),
     {ok, St}.
 
 exometer_newentry(_Entry, St) ->
