@@ -94,17 +94,19 @@ cancel_request(Ref) ->
 
 %% @doc set client options.
 %% Options are:
-%% - `async': to fetch the response asynchronously
-%% - `{async, once}': to receive the response asynchronously once time.
-%% To receive the next message use the function `cloudi_x_hackney:stream_next/1'.
-%% - `{stream_to, pid()}': to set the pid where the messages of an
-%% asynchronous response will be sent.
-%% - `{follow_redirect, bool()}' : if true a redirection will be
-%% followed when the response is received synchronously
-%% - `{force_redirect, bool()}' : if true a 301/302 redirection will be
-%% followed even on POST.
-%% - `{max_redirect, integer()}' the maximum number of redirections that
-%% will be followed
+%% <ul>
+%%  <li>`async': to fetch the response asynchronously</li>
+%%  <li>`{async, once}': to receive the response asynchronously one time.
+%%  To receive the next message use the function `cloudi_x_hackney:stream_next/1'.</li>
+%%  <li>`{stream_to, pid()}': to set the pid where the messages of an
+%%  asynchronous response will be sent.</li>
+%%  <li>`{follow_redirect, bool()}' : if true a redirection will be
+%%  followed when the response is received synchronously</li>
+%%  <li>`{force_redirect, bool()}' : if true a 301/302 redirection will be
+%%  followed even on POST.</li>
+%%  <li>`{max_redirect, integer()}' the maximum number of redirections that
+%%  will be followed</li>
+%% </ul>
 -spec setopts(client_ref(), list()) -> ok.
 setopts(Ref, Options) ->
   cloudi_x_hackney_manager:get_state(Ref, fun(State) ->
@@ -240,11 +242,12 @@ request(Method, URL, Headers, Body) ->
 %%          <li>`insecure': to perform "insecure" SSL connections and
 %%          transfers without checking the certificate</li>
 %%          <li>`{checkout_timeout, infinity | integer()}': timeout used when
-%%          checking out a socket from the pool, in milliseconds. Default is 8000</li>
+%%          checking out a socket from the pool, in milliseconds.
+%%          By default is equal to connect_timeout</li>
 %%          <li>`{connect_timeout, infinity | integer()}': timeout used when
 %%          establishing a connection, in milliseconds. Default is 8000</li>
 %%          <li>`{recv_timeout, infinity | integer()}': timeout used when
-%%          receiving a connection. Default is 5000</li>
+%%          receiving data over a connection. Default is 5000</li>
 %%      </ul>
 %%
 %%      <blockquote>Note: if the response is async, only
@@ -271,17 +274,19 @@ request(Method, URL, Headers, Body) ->
 %%      </li>
 %%  </ul>
 %%
-%%  <bloquote>Note: instead of doing `cloudi_x_hackney:request(Method, ...)' you can
+%%  <blockquote>Note: instead of doing `cloudi_x_hackney:request(Method, ...)' you can
 %%  also do `cloudi_x_hackney:Method(...)' if you prefer to use the REST
-%%  syntax.</bloquote>
+%%  syntax.</blockquote>
 %%
 %%  Return:
 %%  <ul>
 %%  <li><code>{ok, ResponseStatus, ResponseHeaders}</code>: On HEAD
 %%  request if the response succeeded.</li>
-%%  <li><code>{ok, ResponseStatus, ResponseHeaders, Ref}</code>: when
+%%  <li><code>{ok, ResponseStatus, ResponseHeaders, Ref}</code>: When
 %%  the response succeeded. The request reference is used later to
 %%  retrieve the body.</li>
+%%  <li><code>{ok, ResponseStatus, ResponseHeaders, Body}</code>: When the
+%%  option `with_body' is set to true and the response succeeded.</li>
 %%  <li><code>{ok, Ref}</code> Return the request reference when you
 %%  decide to stream the request. You can use the returned reference to
 %%  stream the request body and continue to handle the response.</li>
@@ -293,6 +298,7 @@ request(Method, URL, Headers, Body) ->
 %%  </ul>
 -spec request(term(), url() | binary() | list(), list(), term(), list())
     -> {ok, integer(), list(), client_ref()}
+  | {ok, integer(), list(), binary()}
   | {ok, integer(), list()}
   | {ok, client_ref()}
   | {error, term()}.
@@ -415,7 +421,7 @@ finish_send_body(Ref) ->
 %% <li>`{part, Name}': to start sending a part without length in a streaming
 %% fashion</li>
 %% <li>`{part, Name, ExtraHeader}': to start sending a part without
-%% lengthin a streaming  fashion</li>
+%% length in a streaming  fashion</li>
 %% <li>`{part_bin, Bin}': To send part of part</li>
 %% <li>`{part, eof}': To notify the end of the part </li>
 %% <li>`{mp_mixed, Name, MixedBoundary}': To notify we start a part with a a mixed
@@ -553,7 +559,7 @@ resume_stream(Ref) ->
                                                end).
 
 %% @doc stop to receive asynchronously.
--spec stop_async(client_ref()) -> ok | {error, req_not_found} | {error, term()}.
+-spec stop_async(client_ref()) -> {ok, client_ref()} | {error, req_not_found} | {error, term()}.
 stop_async(Ref) ->
   cloudi_x_hackney_manager:stop_async_response(Ref).
 
@@ -789,7 +795,8 @@ redirect(Client0, {Method, NewLocation, Headers, Body}) ->
   RedirectUrl = cloudi_x_hackney_url:parse_url(NewLocation),
   #cloudi_x_hackney_url{transport=RedirectTransport,
                host=RedirectHost,
-               port=RedirectPort}=RedirectUrl,
+               port=RedirectPort,
+               netloc=RedirectNetloc}=RedirectUrl,
 
   #client{transport=Transport,
           host=Host,
@@ -818,6 +825,7 @@ redirect(Client0, {Method, NewLocation, Headers, Body}) ->
   Client2 = Client1#client{transport=RedirectTransport,
                            host=RedirectHost,
                            port=RedirectPort,
+                           netloc=RedirectNetloc,
                            options=Opts},
 
   %% send a request to the new location
@@ -880,13 +888,12 @@ absolute_url(RelativeUrl, #client{transport=T, host=Host, port=Port,
                   _       -> <<Path/binary, "/", RelativeUrl/binary>>
                 end
             end,
-  Parsed = cloudi_x_hackney_url:normalize(#cloudi_x_hackney_url{scheme=Scheme,
-                                              host=Host,
-                                              port=Port,
-                                              netloc=Netloc,
-                                              path=NewPath}),
-  cloudi_x_hackney_url:unparse_url(Parsed).
 
+  cloudi_x_hackney_url:unparse_url(#cloudi_x_hackney_url{scheme=Scheme,
+                                       host=Host,
+                                       port=Port,
+                                       netloc=Netloc,
+                                       path=NewPath}).
 
 %% handle send response
 reply({ok, Data, NState}, _State) ->
