@@ -8,7 +8,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2018 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2014-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -28,13 +28,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2014-2018 Michael Truog
-%%% @version 1.7.3 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2014-2021 Michael Truog
+%%% @version 2.0.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_http_client).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(cloudi_service).
 
@@ -71,7 +71,6 @@
 %% cloudi_service callbacks
 -export([cloudi_service_init/4,
          cloudi_service_handle_request/11,
-         cloudi_service_handle_info/3,
          cloudi_service_terminate/3]).
 
 -include_lib("cloudi_core/include/cloudi_logger.hrl").
@@ -84,10 +83,35 @@
 
 % supported clients
 -define(MODULE_INETS, httpc). % Erlang/OTP inets HTTP client
+-define(MODULE_HACKNEY, cloudi_x_hackney).
+
+-define(CLIENT_DEBUG_START(Level),
+    if
+        Level =:= off ->
+            undefined;
+        true ->
+            cloudi_timestamp:microseconds_monotonic()
+    end).
+
+-define(CLIENT_DEBUG_END(Level, HttpCode, Method,
+                         HeadersIncoming, Request,
+                         HeadersOutgoing, Response,
+                         RequestStartMicroSec),
+    ?LOG(Level,
+         "~p ~s ~p ms~n"
+         "headers__in(~p)~n"
+         "request__in(~p)~n"
+         "headers_out(~p)~n"
+         "request_out(~p)",
+         [HttpCode, Method,
+          (cloudi_timestamp:microseconds_monotonic() -
+           RequestStartMicroSec) / 1000.0,
+          HeadersIncoming, Request,
+          HeadersOutgoing, Response])).
 
 -record(state,
     {
-        module :: ?MODULE_INETS,
+        module :: ?MODULE_INETS | ?MODULE_HACKNEY,
         profile,
         input_type :: external | internal,
         debug_level :: off | trace | debug | info | warn | error | fatal,
@@ -628,9 +652,19 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
             end,
             {?MODULE_INETS,
              ["head", "get", "put", "post", "trace", "options", "delete"],
-             Profile1}
+             Profile1};
+        Client =:= hackney ->
+            Pool = if
+                Profile0 =:= undefined ->
+                    default;
+                true ->
+                    erlang:list_to_atom(Profile0)
+            end,
+            {?MODULE_HACKNEY,
+             ["head", "get", "put", "post", "trace", "options", "delete"],
+             Pool}
     end,
-    false = lists:member($*, Prefix),
+    false = cloudi_service_name:pattern(Prefix),
     [cloudi_service:subscribe(Dispatcher, [$/ | Method]) || Method <- Methods],
     true = ((DebugLevel =:= trace) orelse
             (DebugLevel =:= debug) orelse
@@ -652,7 +686,8 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
                 content_type_lookup = ContentTypeLookup,
                 prefix_length = erlang:length(Prefix)}}.
 
-cloudi_service_handle_request(_Type, Name, _Pattern, RequestInfo, Request,
+cloudi_service_handle_request(_RequestType, Name, _Pattern,
+                              RequestInfo, Request,
                               Timeout, _Priority, _TransId, _Pid,
                               #state{module = Module,
                                      profile = Profile,
@@ -661,7 +696,7 @@ cloudi_service_handle_request(_Type, Name, _Pattern, RequestInfo, Request,
                                      content_type_lookup = ContentTypeLookup,
                                      prefix_length = PrefixLength} = State,
                               _Dispatcher) ->
-    RequestStartMicroSec = client_debug_start(DebugLevel),
+    RequestStartMicroSec = ?CLIENT_DEBUG_START(DebugLevel),
     [$/ | Method] = cloudi_string:uppercase(lists:nthtail(PrefixLength, Name)),
     HeadersIncoming = headers_request(RequestInfo, InputType),
     {HttpCode,
@@ -669,16 +704,13 @@ cloudi_service_handle_request(_Type, Name, _Pattern, RequestInfo, Request,
      Response} = client_request(Module, Profile, Method,
                                 HeadersIncoming, Request, Timeout,
                                 ContentTypeLookup),
-    client_debug_end(DebugLevel, HttpCode, Method,
-                     HeadersIncoming, Request,
-                     HeadersOutgoing, Response, RequestStartMicroSec),
+    ?CLIENT_DEBUG_END(DebugLevel, HttpCode, Method,
+                      HeadersIncoming, Request,
+                      HeadersOutgoing, Response,
+                      RequestStartMicroSec),
     {reply, HeadersOutgoing, Response, State}.
 
-cloudi_service_handle_info(Request, State, _Dispatcher) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
-
-cloudi_service_terminate(_Reason, _Timeout, #state{}) ->
+cloudi_service_terminate(_Reason, _Timeout, _State) ->
     ok.
 
 %%%------------------------------------------------------------------------
@@ -765,44 +797,29 @@ header_content_type(Headers) ->
             cloudi_string:beforel($;, Value, input)
     end.
 
-url_string({HostName, <<"80">>}, URL) ->
+url_string({HostName, <<"80">>}, URLPath) ->
     "http://" ++ erlang:binary_to_list(HostName) ++
-    erlang:binary_to_list(URL);
-url_string({HostName, <<"443">>}, URL) ->
+    erlang:binary_to_list(URLPath);
+url_string({HostName, <<"443">>}, URLPath) ->
     "https://" ++ erlang:binary_to_list(HostName) ++
-    erlang:binary_to_list(URL);
-url_string({HostName, Port}, URL) ->
+    erlang:binary_to_list(URLPath);
+url_string({HostName, Port}, URLPath) ->
     "http://" ++ erlang:binary_to_list(HostName) ++
     ":" ++ erlang:binary_to_list(Port) ++
-    erlang:binary_to_list(URL).
+    erlang:binary_to_list(URLPath).
 
 client_request(?MODULE_INETS, Profile, Method0,
                HeadersIncoming0, Request, Timeout,
                ContentTypeLookup) ->
-    Method1 = if
-        Method0 == "HEAD" ->
-            head;
-        Method0 == "GET" ->
-            get;
-        Method0 == "PUT" ->
-            put;
-        Method0 == "POST" ->
-            post;
-        Method0 == "TRACE" ->
-            trace;
-        Method0 == "OPTIONS" ->
-            options;
-        Method0 == "DELETE" ->
-            delete
-    end,
     case headers_request_filter(HeadersIncoming0) of
-        {ok, Host, URLPath, HeadersIncoming1} ->
+        {ok, Host, URLPath, HeadersIncomingN} ->
+            MethodN = method_atom(Method0),
             URL = url_string(headers_request_filter_host(Host), URLPath),
             RequestHeaders = [{erlang:binary_to_list(Kin),
                                erlang:binary_to_list(Vin)} ||
-                              {Kin, Vin} <- HeadersIncoming1],
+                              {Kin, Vin} <- HeadersIncomingN],
             ClientRequest = if
-                Method1 =:= get ->
+                MethodN =:= get ->
                     {URL, RequestHeaders};
                 true ->
                     ContentTypeN = case header_content_type(RequestHeaders) of
@@ -819,7 +836,7 @@ client_request(?MODULE_INETS, Profile, Method0,
                     end,
                     {URL, RequestHeaders, ContentTypeN, Request}
             end,
-            case ?MODULE_INETS:request(Method1, ClientRequest,
+            case ?MODULE_INETS:request(MethodN, ClientRequest,
                                        [{autoredirect, false},
                                         {timeout, Timeout}],
                                        [{body_format, binary}], Profile) of
@@ -831,7 +848,43 @@ client_request(?MODULE_INETS, Profile, Method0,
                     HeadersOutgoing1 = [{<<"status">>,
                                          erlang:integer_to_binary(StatusCode)} |
                                         HeadersOutgoing0],
-                    {StatusCode, HeadersOutgoing1, Response};
+                    HeadersOutgoingN = lists:keysort(1, HeadersOutgoing1),
+                    {StatusCode, HeadersOutgoingN, Response};
+                {error, _} = Error ->
+                    {undefined, <<>>, Error}
+            end;
+        {error, _} = Error ->
+            {undefined, <<>>, Error}
+    end;
+client_request(?MODULE_HACKNEY, Profile, Method0,
+               HeadersIncoming0, Request, Timeout,
+               _ContentTypeLookup) ->
+    case headers_request_filter(HeadersIncoming0) of
+        {ok, Host, URLPath, HeadersIncomingN} ->
+            MethodN = method_atom(Method0),
+            URL = url_string(headers_request_filter_host(Host), URLPath),
+            RequestHeaders = HeadersIncomingN,
+            RequestBody = if
+                MethodN =:= get ->
+                    <<>>;
+                true ->
+                    Request
+            end,
+            case ?MODULE_HACKNEY:request(MethodN, URL,
+                                         RequestHeaders, RequestBody,
+                                         [with_body,
+                                          {connect_timeout, Timeout},
+                                          {recv_timeout, Timeout},
+                                          {pool, Profile}]) of
+                {ok, StatusCode, ResponseHeaders, Response} ->
+                    HeadersOutgoing0 = [{cloudi_string:lowercase(Kout),
+                                         Vout} ||
+                                        {Kout, Vout} <- ResponseHeaders],
+                    HeadersOutgoing1 = [{<<"status">>,
+                                         erlang:integer_to_binary(StatusCode)} |
+                                        HeadersOutgoing0],
+                    HeadersOutgoingN = lists:keysort(1, HeadersOutgoing1),
+                    {StatusCode, HeadersOutgoingN, Response};
                 {error, _} = Error ->
                     {undefined, <<>>, Error}
             end;
@@ -839,40 +892,13 @@ client_request(?MODULE_INETS, Profile, Method0,
             {undefined, <<>>, Error}
     end.
 
-client_debug_log(trace, Message, Args) ->
-    ?LOG_TRACE(Message, Args);
-client_debug_log(debug, Message, Args) ->
-    ?LOG_DEBUG(Message, Args);
-client_debug_log(info, Message, Args) ->
-    ?LOG_INFO(Message, Args);
-client_debug_log(warn, Message, Args) ->
-    ?LOG_WARN(Message, Args);
-client_debug_log(error, Message, Args) ->
-    ?LOG_ERROR(Message, Args);
-client_debug_log(fatal, Message, Args) ->
-    ?LOG_FATAL(Message, Args).
-
-client_debug_start(off) ->
-    undefined;
-client_debug_start(_) ->
-    cloudi_timestamp:microseconds_monotonic().
-
-client_debug_end(off, _, _, _, _, _, _, _) ->
-    undefined;
-client_debug_end(Level, HttpCode, Method,
-                 HeadersIncoming, Request,
-                 HeadersOutgoing, Response, RequestStartMicroSec) ->
-    client_debug_log(Level,
-                     "~p ~s ~p ms~n"
-                     "headers__in(~p)~n"
-                     "request__in(~p)~n"
-                     "headers_out(~p)~n"
-                     "request_out(~p)",
-                     [HttpCode, Method,
-                      (cloudi_timestamp:microseconds_monotonic() -
-                       RequestStartMicroSec) / 1000.0,
-                      HeadersIncoming, Request,
-                      HeadersOutgoing, Response]).
+method_atom("HEAD") -> head;
+method_atom("GET") -> get;
+method_atom("PUT") -> put;
+method_atom("POST") -> post;
+method_atom("TRACE") -> trace;
+method_atom("OPTIONS") -> options;
+method_atom("DELETE") -> delete.
 
 result({{ok, {error, _} = Error}, NewAgent}) ->
     {Error, NewAgent};
