@@ -30,7 +30,7 @@
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2012-2020 Michael Truog
-%%% @version 1.8.1 {@date} {@time}
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_http_cowboy1_handler).
@@ -112,17 +112,21 @@ init(_Transport, Req,
     {ok, Req, State}.
 
 handle(Req0,
-       #cowboy1_state{output_type = OutputType,
-                     content_type_forced = ContentTypeForced,
-                     content_types_accepted = ContentTypesAccepted,
-                     set_x_forwarded_for = SetXForwardedFor,
-                     status_code_timeout = StatusCodeTimeout,
-                     query_get_format = QueryGetFormat,
-                     use_host_prefix = UseHostPrefix,
-                     use_client_ip_prefix = UseClientIpPrefix,
-                     use_x_method_override = UseXMethodOverride,
-                     use_method_suffix = UseMethodSuffix
-                     } = State) ->
+       #cowboy1_state{
+           output_type = OutputType,
+           content_type_forced = ContentTypeForced,
+           content_types_accepted = ContentTypesAccepted,
+           content_security_policy = ContentSecurityPolicy,
+           content_security_policy_report = ContentSecurityPolicyReport,
+           set_x_forwarded_for = SetXForwardedFor,
+           set_x_xss_protection = SetXXSSProtection,
+           set_x_content_type_options = SetXContentTypeOptions,
+           status_code_timeout = StatusCodeTimeout,
+           query_get_format = QueryGetFormat,
+           use_host_prefix = UseHostPrefix,
+           use_client_ip_prefix = UseClientIpPrefix,
+           use_x_method_override = UseXMethodOverride,
+           use_method_suffix = UseMethodSuffix} = State) ->
     RequestStartMicroSec = ?LOG_WARN_APPLY(fun request_time_start/0, []),
     {MethodHTTP, Req1} = cloudi_x_cowboy1_req:method(Req0),
     {HeadersIncoming0, Req2} = cloudi_x_cowboy1_req:headers(Req1),
@@ -261,7 +265,11 @@ handle(Req0,
                     {HttpCode,
                      Req} = handle_response(NameIncoming, HeadersOutgoing,
                                             Response, ReqN0, OutputType,
-                                            ContentTypeForced),
+                                            ContentTypeForced,
+                                            SetXContentTypeOptions,
+                                            SetXXSSProtection,
+                                            ContentSecurityPolicy,
+                                            ContentSecurityPolicyReport),
                     ?LOG_TRACE_APPLY(fun request_time_end_success/5,
                                      [HttpCode, MethodHTTP,
                                       NameIncoming, NameOutgoing,
@@ -913,6 +921,14 @@ headers_external_outgoing(ResponseInfo)
     when is_binary(ResponseInfo) ->
     cloudi_response_info:key_value_parse(ResponseInfo, list).
 
+header_set_if_not(Key, Value, Headers) ->
+    case lists:keyfind(Key, 1, Headers) of
+        {_, _} ->
+            Headers;
+        false ->
+            [{Key, Value} | Headers]
+    end.
+
 get_query_string_external(QsVals) ->
     cloudi_request_info:key_value_new(QsVals, text_pairs).
 
@@ -1206,7 +1222,9 @@ handle_request_multipart_receive([_ | _] = TransIdList, Req,
     end.
 
 handle_response(NameIncoming, HeadersOutgoing0, Response,
-                ReqN, OutputType, ContentTypeForced) ->
+                Req0, OutputType, ContentTypeForced,
+                SetXContentTypeOptions, SetXXSSProtection,
+                ContentSecurityPolicy, ContentSecurityPolicyReport) ->
     ResponseBinary = if
         (((OutputType =:= external) orelse
           (OutputType =:= internal)) andalso
@@ -1218,7 +1236,7 @@ handle_response(NameIncoming, HeadersOutgoing0, Response,
          is_list(Response)) ->
             erlang:iolist_to_binary(Response)
     end,
-    {HttpCode, HeadersOutgoingN} = case lists:keytake(<<"status">>, 1,
+    {HttpCode, HeadersOutgoing2} = case lists:keytake(<<"status">>, 1,
                                                       HeadersOutgoing0) of
         false ->
             {200, HeadersOutgoing0};
@@ -1227,9 +1245,9 @@ handle_response(NameIncoming, HeadersOutgoing0, Response,
             {erlang:binary_to_integer(hd(binary:split(Status, <<" ">>))),
              HeadersOutgoing1}
     end,
-    ResponseHeadersOutgoing = if
-        HeadersOutgoingN =/= [] ->
-            HeadersOutgoingN;
+    HeadersOutgoing3 = if
+        HeadersOutgoing2 =/= [] ->
+            HeadersOutgoing2;
         ContentTypeForced =/= undefined ->
             [{<<"content-type">>, ContentTypeForced}];
         true ->
@@ -1259,11 +1277,64 @@ handle_response(NameIncoming, HeadersOutgoing0, Response,
                     end
             end
     end,
-    {ok, Req} = cloudi_x_cowboy1_req:reply(HttpCode,
-                                          ResponseHeadersOutgoing,
-                                          ResponseBinary,
-                                          ReqN),
-    {HttpCode, Req}.
+    {ContentTypeHTML,
+     ContentTypeSet} = case lists:keyfind(<<"content-type">>, 1,
+                                          HeadersOutgoing3) of
+        {_,  <<"text/html", _/binary>>} ->
+            {true, true};
+        {_,  <<_/binary>>} ->
+            {false, true};
+        false ->
+            {false, false}
+    end,
+    HeadersOutgoing4 = if
+        ContentTypeSet =:= true ->
+            if
+                SetXContentTypeOptions =:= true ->
+                    header_set_if_not(<<"X-Content-Type-Options">>,
+                                      <<"nosniff">>,
+                                      HeadersOutgoing3);
+                SetXContentTypeOptions =:= false ->
+                    HeadersOutgoing3
+            end;
+        ContentTypeSet =:= false ->
+            HeadersOutgoing3
+    end,
+    HeadersOutgoingN = if
+        ContentTypeHTML =:= true ->
+            HeadersOutgoing5 = if
+                SetXXSSProtection =:= true ->
+                    header_set_if_not(<<"X-XSS-Protection">>,
+                                      <<"0">>,
+                                      HeadersOutgoing4);
+                SetXXSSProtection =:= false ->
+                    HeadersOutgoing4
+            end,
+            HeadersOutgoing6 = if
+                is_binary(ContentSecurityPolicyReport) ->
+                    header_set_if_not(<<"content-security-policy-report-only">>,
+                                      ContentSecurityPolicyReport,
+                                      HeadersOutgoing5);
+                ContentSecurityPolicyReport =:= undefined ->
+                    HeadersOutgoing5
+            end,
+            if
+                is_binary(ContentSecurityPolicy) ->
+                    header_set_if_not(<<"content-security-policy">>,
+                                      ContentSecurityPolicy,
+                                      HeadersOutgoing6);
+                ContentSecurityPolicy =:= undefined ->
+                    HeadersOutgoing6
+            end;
+        ContentTypeHTML =:= false ->
+            HeadersOutgoing4
+    end,
+    ResponseHeadersOutgoing = HeadersOutgoingN,
+    {ok, ReqN} = cloudi_x_cowboy1_req:reply(HttpCode,
+                                            ResponseHeadersOutgoing,
+                                            ResponseBinary,
+                                            Req0),
+    {HttpCode, ReqN}.
 
 service_name_incoming(UseClientIpPrefix, UseHostPrefix, PathRaw, Client, Req0)
     when UseClientIpPrefix =:= true, UseHostPrefix =:= true ->
@@ -1401,12 +1472,12 @@ websocket_disconnect_check({async, WebSocketDisconnectName}, Reason,
                                output_type = OutputType,
                                websocket_state = #websocket_state{
                                    request_info = RequestInfo}}) ->
-    send_async_minimal(Dispatcher, WebSocketDisconnectName,
-                       websocket_disconnect_request_info(Reason,
-                                                         RequestInfo,
-                                                         OutputType),
-                       websocket_disconnect_request(OutputType),
-                       TimeoutAsync, self()),
+    _ = send_async_minimal(Dispatcher, WebSocketDisconnectName,
+                           websocket_disconnect_request_info(Reason,
+                                                             RequestInfo,
+                                                             OutputType),
+                           websocket_disconnect_request(OutputType),
+                           TimeoutAsync, self()),
     ok;
 websocket_disconnect_check({sync, WebSocketDisconnectName}, Reason,
                            #cowboy1_state{
@@ -1415,12 +1486,12 @@ websocket_disconnect_check({sync, WebSocketDisconnectName}, Reason,
                                output_type = OutputType,
                                websocket_state = #websocket_state{
                                    request_info = RequestInfo}}) ->
-    send_sync_minimal(Dispatcher, WebSocketDisconnectName,
-                      websocket_disconnect_request_info(Reason,
-                                                        RequestInfo,
-                                                        OutputType),
-                      websocket_disconnect_request(OutputType),
-                      TimeoutSync, self()),
+    _ = send_sync_minimal(Dispatcher, WebSocketDisconnectName,
+                          websocket_disconnect_request_info(Reason,
+                                                            RequestInfo,
+                                                            OutputType),
+                          websocket_disconnect_request(OutputType),
+                          TimeoutSync, self()),
     ok.
 
 websocket_subscriptions([], _, _) ->
