@@ -10,7 +10,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2014-2019 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2014-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -31,8 +31,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2014-2019 Michael Truog
-%%% @version 1.8.0 {@date} {@time}
+%%% @copyright 2014-2021 Michael Truog
+%%% @version 2.0.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_service_oauth1).
@@ -196,9 +196,10 @@ cloudi_service_init(Args, Prefix, _Timeout, Dispatcher) ->
         TokensClean =:= undefined ->
             ok;
         is_integer(TokensClean), TokensClean > 0, TokensClean =< 4294967 ->
-            erlang:send_after(TokensClean * 1000,
-                              cloudi_service:self(Dispatcher),
-                              {tokens_clean, TokensClean})
+            _ = erlang:send_after(TokensClean * 1000,
+                                  cloudi_service:self(Dispatcher),
+                                  {tokens_clean, TokensClean}),
+            ok
     end,
     true = (is_integer(TokenRequestLength) andalso
             (TokenRequestLength > 0)),
@@ -385,7 +386,7 @@ url(Method, RequestHeaders, URLHost, Host, Request) ->
     case cloudi_key_value:find(<<"host">>, RequestHeaders) of
         {ok, Host} when Method == "GET" ->
             {ok, URLHost ++ erlang:binary_to_list(URLPath) ++ "?" ++
-                 erlang:binary_to_list(cloudi_x_cow1_qs:qs(Request))};
+                 erlang:binary_to_list(cloudi_x_cow_qs:qs(Request))};
         {ok, Host} ->
             {ok, URLHost ++ erlang:binary_to_list(URLPath)};
         {ok, HostInvalid} ->
@@ -712,7 +713,7 @@ callback_merge_qs_filter_request([{K, _} = Entry |
 callback_merge(CallbackURL, CallbackQS, RequestQS, Verifier) ->
     NewCallbackQS = callback_merge_qs_filter_callback(CallbackQS),
     NewRequestQS = callback_merge_qs_filter_request(RequestQS, CallbackQS),
-    QS = cloudi_x_cow1_qs:qs(NewCallbackQS ++ NewRequestQS ++
+    QS = cloudi_x_cow_qs:qs(NewCallbackQS ++ NewRequestQS ++
                    [{<<"oauth_verifier">>, Verifier}]),
     <<CallbackURL/binary, $?, QS/binary>>.
 
@@ -750,7 +751,7 @@ request_authorize(RequestQS, TokenRequest, Timeout,
             end;
         {ok, CallbackURL, CallbackQS} ->
             Callback = callback_merge(CallbackURL,
-                                      cloudi_x_cow1_qs:parse_qs(CallbackQS),
+                                      cloudi_x_cow_qs:parse_qs(CallbackQS),
                                       RequestQS, Verifier),
             case DatabaseModule:token_request_update(Dispatcher, Database,
                                                      TokenRequest, Verifier,
