@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2011-2017 Michael Truog <mjtruog at gmail dot com>
+%%% Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -29,13 +29,13 @@
 %%% FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 %%% DEALINGS IN THE SOFTWARE.
 %%%
-%%% @author Michael Truog <mjtruog [at] gmail (dot) com>
-%%% @copyright 2011-2017 Michael Truog
-%%% @version 1.7.1 {@date} {@time}
+%%% @author Michael Truog <mjtruog at protonmail dot com>
+%%% @copyright 2011-2021 Michael Truog
+%%% @version 2.0.2 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_supool).
--author('mjtruog [at] gmail (dot) com').
+-author('mjtruog at protonmail dot com').
 
 -behaviour(gen_server).
 
@@ -52,14 +52,12 @@
          handle_call/3, handle_cast/2, handle_info/2,
          terminate/2, code_change/3]).
 
--include("cloudi_x_supool_logging.hrl").
-
 -record(state,
     {
         supervisor :: pid(),
         index = 1 :: pos_integer(),
-        pool = undefined :: tuple() | undefined,
-        count = undefined :: pos_integer() | undefined
+        count = undefined :: pos_integer() | undefined,
+        pool = undefined :: tuple() | undefined
     }).
 
 -type options() ::
@@ -148,65 +146,54 @@ init([Supervisor]) ->
     self() ! restart,
     {ok, #state{supervisor = Supervisor}}.
 
-handle_call(get, _, #state{supervisor = Supervisor,
-                           index = I,
-                           pool = Pool,
-                           count = Count} = State) ->
-    Pid = erlang:element(I, Pool),
-    case erlang:is_process_alive(Pid) of
-        true ->
-            {reply, Pid, State#state{index = increment(I, Count)}};
-        false ->
-            NewPids = cloudi_x_supool_sup:which_children(Supervisor),
-            case erlang:length(NewPids) of
-                0 ->
-                    {stop, {error, noproc}, undefined, State};
-                NewCount ->
-                    NewPool = erlang:list_to_tuple(NewPids),
-                    NewI = if I > NewCount -> 1; true -> I end,
-                    {reply, erlang:element(NewI, NewPool),
-                     State#state{index = increment(NewI, NewCount),
-                                 pool = NewPool,
-                                 count = NewCount}}
-            end
-    end;
-
+handle_call(get, _, #state{index = Index,
+                           count = Count,
+                           pool = Pool} = State) ->
+    {Result, IndexNew} = pool_pid(Index, Count, Pool),
+    {reply, Result, State#state{index = IndexNew}};
 handle_call(Request, _, State) ->
-    ?LOG_WARN("Unknown call \"~p\"", [Request]),
-    {stop, lists:flatten(io_lib:format("Unknown call \"~p\"", [Request])),
+    {stop, lists:flatten(io_lib:format("Unknown call \"~w\"", [Request])),
      error, State}.
 
 handle_cast(Request, State) ->
-    ?LOG_WARN("Unknown cast \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, lists:flatten(io_lib:format("Unknown cast \"~w\"", [Request])),
+     State}.
 
 handle_info({start, ChildSpecs}, #state{supervisor = Supervisor} = State) ->
     case cloudi_x_supool_sup:start_children(Supervisor, ChildSpecs) of
         {ok, []} ->
             {stop, {error, noproc}, State};
         {ok, Pids} ->
-            {noreply, State#state{pool = erlang:list_to_tuple(Pids),
-                                  count = erlang:length(Pids)}};
+            {noreply, State#state{count = length(Pids),
+                                  pool = erlang:list_to_tuple(Pids)}};
         {error, _} = Error ->
             {stop, Error, State}
     end;
-
 handle_info(restart, #state{supervisor = Supervisor} = State) ->
     Pids = cloudi_x_supool_sup:which_children(Supervisor),
-    case erlang:length(Pids) of
+    case length(Pids) of
         0 ->
             % pool worker started for the first time
             {noreply, State};
         Count ->
             % pool worker restarted
             {noreply,
-             State#state{pool = erlang:list_to_tuple(Pids),
-                         count = Count}}
+             State#state{count = Count,
+                         pool = erlang:list_to_tuple(Pids)}}
+    end;
+handle_info(update, #state{supervisor = Supervisor} = State) ->
+    Pids = cloudi_x_supool_sup:which_children(Supervisor),
+    case length(Pids) of
+        0 ->
+            {stop, {error, noproc}, State};
+        Count ->
+            {noreply,
+             State#state{count = Count,
+                         pool = erlang:list_to_tuple(Pids)}}
     end;
-
 handle_info(Request, State) ->
-    ?LOG_WARN("Unknown info \"~p\"", [Request]),
-    {noreply, State}.
+    {stop, lists:flatten(io_lib:format("Unknown info \"~w\"", [Request])),
+     State}.
 
 terminate(_, _) ->
     ok.
@@ -218,6 +205,27 @@ code_change(_, State, _) ->
 %%% Private functions
 %%%------------------------------------------------------------------------
 
+pool_pid(I, Count, Pool) ->
+    Pid = element(I, Pool),
+    case erlang:is_process_alive(Pid) of
+        true ->
+            {Pid, increment(I, Count)};
+        false ->
+            self() ! update,
+            pool_pid(increment(I, Count), I, Count, Pool)
+    end.
+
+pool_pid(I, I, _, _) ->
+    {undefined, I};
+pool_pid(I, IndexStart, Count, Pool) ->
+    Pid = element(I, Pool),
+    case erlang:is_process_alive(Pid) of
+        true ->
+            {Pid, increment(I, Count)};
+        false ->
+            pool_pid(increment(I, Count), IndexStart, Count, Pool)
+    end.
+
 increment(Count, Count) ->
     1;
 increment(I, _) ->
