@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2011-2017 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -35,10 +35,12 @@ if __FILE__ == $PROGRAM_NAME
     threads = (0...thread_count).to_a.map{ |i| Thread.new(i){ |thread_index|
         class Task
             def initialize(thread_index)
-                @api = CloudI::API.new(thread_index)
+                @api = nil
+                @thread_index = thread_index
             end
 
             def run
+                @api = CloudI::API.new(@thread_index)
                 @api.subscribe('ruby', method(:request))
 
                 result = @api.poll
@@ -56,7 +58,7 @@ if __FILE__ == $PROGRAM_NAME
             def request(request_type, name, pattern, request_info, request,
                         timeout, priority, trans_id, pid)
                 i = request[0,4].unpack('L')[0]
-                if i == 4294967295
+                if i == 1073741823
                     i = 0
                 else
                     i += 1
@@ -70,13 +72,8 @@ if __FILE__ == $PROGRAM_NAME
                               timeout, priority, trans_id, pid)
             end
         end
-        begin
-            object = Task.new(thread_index)
-            object.run
-        rescue
-            $stderr.puts $!.message
-            $stderr.puts $!.backtrace
-        end
+
+        Task.new(thread_index).run
     }}
     threads.each{ |t| t.join }
 end
